<?php

class Tbp_Dynamic_Content {

	private static $items = array();

	/**
	 * Name of the option that stores Dynamic Content settings
	 *
	 * @type string
	 */
	const FIELD_NAME = '__dc__';

	public static function run() {
		add_filter( 'tf_builder_row', array( __CLASS__, 'tf_builder_row' ) );
		add_filter( 'themify_builder_module_render_vars', array( __CLASS__, 'themify_builder_module_render_vars' ) );
		add_action( 'themify_builder_frontend_enqueue', array( __CLASS__, 'admin_enqueue' ) );
		add_action( 'themify_builder_admin_enqueue', array( __CLASS__, 'admin_enqueue' ), 15 );
		add_action( 'wp_ajax_tpb_get_dynamic_content_fields', array( __CLASS__, 'options' ) );
		add_action( 'themify_builder_background_styling', array( __CLASS__, 'background_styling' ), 10, 4 );

		/* Frontend editor preview */
		add_action( 'wp_ajax_tpb_get_dynamic_content_preview', array( __CLASS__, 'preview' ) );
		add_filter( 'themify_builder_data', array( __CLASS__, 'frontend_editor_preview' ), 10, 2 );
		add_filter( 'themify_builder_load_module_partial', array( __CLASS__, 'themify_builder_load_module_partial' ) );
	}

	private static function register_items( $load_all = false ) {
		if ( $load_all===false && !empty(self::$items)) {
			return;
		}

		$items =  array();
		$base_path = TBP_DIR . 'includes/dynamic-content/';
		$files = scandir ( $base_path );
		foreach ($files as $file) {
			if (pathinfo($file,PATHINFO_EXTENSION) === 'php' ) {
				$name = pathinfo( $file, PATHINFO_FILENAME );
				if ( ! isset( self::$items[ $name ] ) ) {
					include_once $base_path . $file;
					$items[ $name ] = "Tbp_Dynamic_Item_{$name}";
				}
			}
		}
		$files = $file = null;
		foreach ( $items as $id => $class ) {
		    $instance = new $class();
		    /* add this item only if is_available() */
		    if ( $instance->is_available() === true ) {
			    self::$items[ $id ] = $instance;
		    } 
		    else {
			$instance = null;
		    }
		}

		return self::$items;
	}

	public static function get( $id = null ) {
		if ( isset( self::$items[ $id ] ) ) {
			return self::$items[ $id ];
		} else {
			$class = "Tbp_Dynamic_Item_{$id}";
			if ( ! class_exists( $class ) ) {
				$base_path = TBP_DIR . 'includes/dynamic-content/';
				if ( is_file( $base_path . $id . '.php' ) ) {
					include $base_path . $id . '.php';
				}
			}
			if ( class_exists( $class ) ) {
				$instance = new $class();
				if ( $instance->is_available() === true ) {
					self::$items[ $id ] = $instance;
					return $instance;
				}
			}
		}

		return false;
	}

	/**
	 * Returns an assoc array
	 *
	 * @return array
	 */
	public static function get_list() {
		$list = array();
		foreach ( self::$items as $id => $instance ) {
			$list[ $id ] = array(
				'type' => $instance->get_type(),
			);
		}

		return $list;
	}

	/**
	 * Adds inline styles for styling the background image of Builder components
	 *
	 * hooked to "themify_builder_background_styling"
	 */
	public static function background_styling( $builder_id, $settings, $type ) {
		if ( ! isset( $settings[ 'styling' ][ self::FIELD_NAME ] ) || $settings[ 'styling' ][ self::FIELD_NAME ]==='{}' ) {
			return;
		}
		$dc = is_string($settings[ 'styling' ][ self::FIELD_NAME ])?json_decode( $settings[ 'styling' ][ self::FIELD_NAME ], true ):$settings[ 'styling' ][ self::FIELD_NAME ];
		
		if ( ! is_array( $dc ) ) {
			return;
		}

		if ( $type === 'sub_column' ) {
			$type = 'column';
		}
		$element_id = 'tb_' . $settings['element_id'];
		if ( $type === 'row' || $type === 'column' || $type === 'subrow' ) {
			$bg_fields = self::get_background_image_fields( $type );
			$type_selector='.module_' . $type;
		} else {
			$mod_name = $settings['mod_name'];
			$bg_fields = self::get_background_image_fields( $mod_name );
			$type_selector='.module-' . $mod_name;
		}
		if ( empty( $bg_fields )) {
			return;
		}
		$intersect = array_intersect_key($dc,$bg_fields);
		if(empty($intersect)){
			return;
		}
		$dc=null;
		$styles = '';
		$base='';
		if(Tbp_Utils::$isLoop===true){
		    if(class_exists('TB_Advanced_Posts_Module') && TB_Advanced_Posts_Module::$builder_id!==null){
				$builder_id = str_replace( 'tb_', '', TB_Advanced_Posts_Module::$builder_id );
		    }
		    elseif(class_exists('TB_Advanced_Products_Module') && TB_Advanced_Products_Module::$builder_id!==null){
				$builder_id = str_replace( 'tb_', '', TB_Advanced_Products_Module::$builder_id );
		    }
		}
		else{
		    $base='.themify_builder';
		}
		$base.='.themify_builder_content-'.$builder_id;
		foreach ( $intersect as $key => $options ) {
			if ( $value = self::get_value( $options ) ) {
				$selector = $base;
				if(Tbp_Utils::$isLoop===true){
				    $selector.=' .post-'.get_the_ID();
				}
				$selector.=" {$type_selector}.{$element_id}";
				if(is_string($bg_fields[ $key ])){
				    $selector.=$bg_fields[ $key ];
				}
				else{
				    $selector.=implode(',',$bg_fields[ $key ]);
				}
			    $styles.= $selector . '{background-image:url("' . $value . '")}';
			
			}
		}
		if ( $styles!=='' ) {
		    echo '<style class="tbp_dc_styles">' , $styles , '</style>';
		}
	}

	/**
	 * Loops through a component styling definition to find all background-image fields
	 *
	 * @return array
	 */
	private static function get_background_image_fields( $type ) {
		if ( empty( $type ) ) {
			return array();
		}
		static $cacheIDs = array();
		if ( ! isset( $cacheIDs[ $type ] ) ) {
			$list = array();
			if ( $type === 'row' || $type === 'column' || $type === 'subrow' ) {
				$list = array( 'background_image' => '' );
				$inner_select='>div.';
				$inner_select .= $type === 'column' ? 'tb-column-inner' : $type . '_inner';
				$list['background_image_inner'] = array( $inner_select );
				if ( $type === 'column' && Themify_Builder::$frontedit_active === true ) {
				    $list['background_image_inner'][]= '>.tb_holder';
				}
			} 
			else {
				/* load the module first */
				if ( ! isset( Themify_Builder_Model::$modules[ $type ] ) ) {
					Themify_Builder_Component_Module::load_modules( $type );
				}
				/* safety check */
				if ( isset( Themify_Builder_Model::$modules[ $type ] ) ) {
					$module = Themify_Builder_Model::$modules[ $type ];
					$array = $module->get_form_settings( true );

					$iterator  = new RecursiveArrayIterator( $array );
					$recursive = new RecursiveIteratorIterator( $iterator, RecursiveIteratorIterator::SELF_FIRST );
					foreach ( $recursive as  $value ) {
						if ( isset( $value['prop'], $value['id'] ) && !isset($value['ishover']) && $value['prop'] === 'background-image' && ($value['label']==='bg' ||  $value['label']==='b_i')  && ($value['type']==='image' ||  $value['type']==='imageGradient')) {
							$list[ $value['id'] ] = $value['selector'];
						}
					}
				}
			}
			$cacheIDs[ $type ] = $list;
		}

		return $cacheIDs[ $type ];
	}

	/**
	 * Update Builder data in frontend preview
	 * uses self::do_replace() and retains the DC data
	 */
	public static function frontend_editor_preview( $data, $post_id ) {
		if ( $post_id === Themify_Builder::$builder_active_id && Themify_Builder_Model::is_front_builder_activate()) {
			if ( ! empty( $data ) ) {
				foreach ( $data as $row_index => &$row ) {
					if ( isset( $row['styling'] ) ) {
						$replace = self::do_replace( $row['styling'], [ 'unset_dc_data' => false ] );
						if ( is_array( $replace ) ) {
							$row['styling'] = $replace;
						}
					}

					if ( ! empty( $row['cols'] ) ) {
						foreach ( $row['cols'] as $column_index => &$column ) {

							if ( isset( $column['styling'] ) ) {
								$replace = self::do_replace( $column['styling'], [ 'unset_dc_data' => false ] );
								if ( is_array( $replace ) ) {
									$column['styling'] = $replace;
								}
							}

							if ( ! empty( $column['modules'] ) ) {
								foreach ( $column['modules'] as $module_index => &$module ) {

									if ( isset( $module['styling'] ) ) {
										$replace = self::do_replace( $module['styling'], [ 'unset_dc_data' => false ] );
										if ( is_array( $module['styling'] ) ) {
											$module['styling'] = $replace;
										}
									}

									/* subrows */
									if ( ! empty( $module['cols'] ) ) {
										foreach ( $module['cols'] as $sub_column_index => &$sub_column ) {

											if ( isset( $sub_column['styling'] ) ) {
												$replace = self::do_replace( $sub_column['styling'], [ 'unset_dc_data' => false ] );
												if ( is_array( $replace ) ) {
													$sub_column['styling'] = $replace;
												}
											}

											if ( ! empty( $sub_column['modules'] ) ) {
												foreach ( $sub_column['modules'] as $sub_column_module_index => &$sub_column_module ) {
													$replace = self::do_replace( $sub_column_module['mod_settings'], [ 'unset_dc_data' => false ] );
													if ( is_array( $replace ) ) {
														$sub_column_module['mod_settings'] = $replace;
													}
												}
											}
										}
									} 
									elseif ( isset( $module['mod_settings'] ) ) {
										$replace = self::do_replace( $module['mod_settings'], [
											'ignore_disable_subrow_condition' => true,
											'unset_dc_data' => false,
										] );
										if ( is_array( $replace ) ) {
											$module['mod_settings'] = $replace;
										}
									}
								}
							}
						}
					}
				}
			}
		}

		return $data;
	}

	/**
	 * Fix preview in load_module_partial
	 *
	 * @return array
	 */
	public static function themify_builder_load_module_partial($batch ) {
		if (! empty( $batch ) ) {
			if(isset($_POST['post_type']) && $_POST['post_type']==='product' && themify_is_woocommerce_active()){
			    $query = Tbp_Utils::get_wc_actual_query();
			}
			else{
			    $query = Tbp_Utils::get_actual_query();
			}

			if ( $query !== null && $query->have_posts() ) {
				$query->the_post();
			}

			foreach ( $batch as &$component ) {
			    if(isset($component['elType'])){
				$type = $component['elType'];
				$key = $type === 'module' ? 'mod_settings' : 'styling';
				if ( ! empty( $component[ $key ] ) ) {
					$replace = self::do_replace( $component[ $key ], [ 'unset_dc_data' => false ] );
					if ( is_array( $replace ) ) {
						$component[ $key ] = $replace;
					}
				}
			    }
			}
		}

		return $batch;
	}

	/**
	 * Loop through a row
	 *
	 * @return array
	 */
	public static function tf_builder_row( $row ) {
		if ( isset( $row['styling'] ) ) {
			$replace = self::do_replace( $row['styling'], [ 'type' => 'row' ] );
			if ( $replace === '__disable_row__' ) {
				return array();
			} else {
				$row['styling'] = $replace;
			}
		}

		if ( ! empty( $row['cols'] ) ) {
			foreach ( $row['cols'] as $column_index => &$column ) {

				if ( isset( $column['styling'] ) ) {
					$replace = self::do_replace( $column['styling'], [ 'type' => 'column' ] );
					if ( $replace === '__disable_row__' ) {
						return array();
					} else {
						$column['styling'] = $replace;
					}
				}

				if ( ! empty( $column['modules'] ) ) {
					foreach ( $column['modules'] as $module_index => &$module ) {
						if ( isset( $module['mod_settings'] ) ) {
							$replace = self::do_replace( $module['mod_settings'], [ 'type' => $module['mod_name'] ] );
							if ( $replace === '__disable_row__' ) {
								return array();
							} 
							elseif ( $replace === '__disable_subrow__' ) {
								unset( $row['cols'][ $column_index ]['modules'][ $module_index ] );
								continue;
							}
							else {
								$module['mod_settings'] = $replace;
							}
						}

						/* subrows */
						if ( ! empty( $module['cols'] ) ) {
							foreach ( $module['cols'] as $sub_column_index => &$sub_column ) {

								if ( isset( $sub_column['styling'] ) ) {
									$replace = self::do_replace( $sub_column['styling'], [ 'type' => 'column' ] );
									if ( $replace === '__disable_row__' ) {
										return array();
									} elseif ( $replace === '__disable_subrow__' ) {
										unset( $row['cols'][ $column_index ]['modules'][ $module_index ] );
										continue 2;
									} else {
										$sub_column['styling'] = $replace;
									}
								}

								if ( ! empty( $sub_column['modules'] ) ) {
									foreach ( $sub_column['modules'] as $sub_column_module_index => &$sub_column_module ) {
									    if(isset($sub_column_module['mod_settings'])){
										$replace = self::do_replace( $sub_column_module['mod_settings'], [ 'type' => $sub_column_module['mod_name'] ] );
										if ( $replace === '__disable_module__' ) {
											unset( $row['cols'][ $column_index ]['modules'][ $module_index ]['cols'][ $sub_column_index ]['modules'][ $sub_column_module_index ] );
										} elseif ( $replace === '__disable_row__' ) {
											// hide entire row
											return array();
										} elseif ( $replace === '__disable_subrow__' ) {
											// hide subrow
											unset( $row['cols'][ $column_index ]['modules'][ $module_index ] );
										} else {
											$sub_column_module['mod_settings'] = $replace;
										}
									    }
									}
								}
							}
						} 
						elseif ( isset( $module['mod_settings'] ) ) {
							$replace = self::do_replace( $module['mod_settings'], [
								'ignore_disable_subrow_condition' => true,
								'type' => $module['mod_name']
							] );
							if ( $replace === '__disable_module__' ) {
								/* hide the module */
								unset( $row['cols'][ $column_index ]['modules'][ $module_index ] );
							} elseif ( $replace === '__disable_row__' ) {
								/* hide the entire row */
								return array();
							} else {
								$module['mod_settings'] = $replace;
							}
						}
					}
				}
			}
		}
		

		return $row;
	}

	/**
	 * Filter module settings in preview
	 *
	 * @return array
	 */
	public static function themify_builder_module_render_vars( $vars ) {
		if ( ! empty( $_POST['action'] ) && $_POST['action'] === 'tb_save_data' ) {
			return $vars;
		}

		return  self::do_replace( $vars );
	}

	/**
	 * Parse DC settings and replace the module settings in $vars with their values.
	 *
	 * @return array|string
	 */
	public static function do_replace( $vars, $args = [] ) {
		if ( ! isset( $vars[ self::FIELD_NAME ] ) || $vars[ self::FIELD_NAME ]==='{}' )
			return $vars;
		$fields = is_string( $vars[ self::FIELD_NAME ] ) ? json_decode( $vars[ self::FIELD_NAME ], true ) : $vars[ self::FIELD_NAME ];
	
		if ( empty( $fields ) || ! is_array( $fields ) ) {
			return $vars;
		}

		$args = wp_parse_args( $args, [
			'ignore_disable_subrow_condition' => false,
			'unset_dc_data' => true, /* whether to remove the DC settings after parsing */
			'type' => '',
		] );

		foreach ( $fields as $key => $options ) {
			if ( ! isset( $options['item'] ) || isset( $options['repeatable'] ) ) {
			    if ( isset( $vars[ $key ] ) && is_array( $vars[ $key ] ) ) {
					if ( $args['unset_dc_data'] ) {
						unset( $options['repeatable'], $options['o'] );
					}
					// loop through repeatable items
					if ( ! empty( $options ) && is_array( $options ) ) {
						foreach ( $options as $i => $items ) {
							if ( ! empty( $items ) ) {
								foreach ( $items as $field_name => $field_options ) {
									if ( isset( $field_options['item'] ) ) {
										$value = self::get_value( $field_options );
										if ( in_array( $value, array( '__disable_module__', '__disable_row__' ) ,true) ) {
											return $value;
										} 
										elseif ( '__disable_subrow__' === $value ) {
											if ( $args['ignore_disable_subrow_condition'] ) {
												// replace field with empty string
												$vars[ $key ][ $i ][ $field_name ] = '';
											} else {
												// flag the subrow to be removed
												return '__disable_subrow__';
											}
										} else {
											$vars[ $key ][ $i ][ $field_name ] = $value;
										}
									}
								}
							}
						}
					}
			    }
			} else {
				$value = self::get_value( $options );
				if ( in_array( $value, array( '__disable_module__', '__disable_row__' ),true ) ) {
					// special flags, return the string instead of replacing
					return $value;
				} 
				if ( '__disable_subrow__' === $value ) {
					if ( $args['ignore_disable_subrow_condition'] ) {
					    // replace field with empty string
					     $vars[ $key ] = '';
					} 
					else {
					    // flag the subrow to be removed
					    return '__disable_subrow__';
					}
				} else {
					$vars[ $key ] = $value;
				}
			}
		}

		if ( $args['unset_dc_data'] ) {
			/* a list of DC fields needed for dynamic background styling */
			$dynamic_bg_fields = array_intersect_key( $vars[ self::FIELD_NAME ], self::get_background_image_fields( $args['type'] ) );
			/* clear the DC settings so it's not parsed multiple times */
			if ( ! empty( $dynamic_bg_fields ) ) {
				/* keep the background styling fields to be parsed in background_styling hook */
				$vars[ self::FIELD_NAME ] = $dynamic_bg_fields;
			} else {
				unset( $vars[ self::FIELD_NAME ] );
			}
		}

		return $vars;
	}

	/**
	 * Get value from saved DC settings
	 *
	 * Calls Tbp_Dynamic_Content::get_value for $options['item']
	 */
	private static function get_value( $options ) {
		if ( isset( $options['item'] ) && ( $item = self::get( $options['item'] ) ) ) {
			unset( $options['item'] );

			$value = $item->get_value( $options );

			if ( empty( $value ) ) {
				if ( ! empty( $options['condition'] ) && ! Themify_Builder::$frontedit_active ) {
					if ( $options['condition'] === 'show_text' ) {
						if ( isset( $options['__fb_val'] ) ) {
							return $options['__fb_val'];
						}
					} 
					elseif ( $options['condition'] === 'hide_module' ) {
						return '__disable_module__'; // flag to remove the module
					} 
					if ( $options['condition'] === 'hide_row' ) {
						return '__disable_row__'; // flag to remove the row
					} 
					if ( $options['condition'] === 'hide_subrow' ) {
						return '__disable_subrow__'; // flag to remove the subrow
					}
				}
			} else {
				// general formatting for various field types
				if ( isset( $options['text_before'] ) ) {
					$value = $options['text_before'] . $value;
				}
				if ( isset( $options['text_after'] ) ) {
					$value .= $options['text_after'];
				}
				if ( ! empty( $options['uri_scheme'] ) ) {
					$value = $options['uri_scheme'] . ':' . $value;
				}
			}

			return $value;
		}

		return null;
	}

	public static function admin_enqueue() {
	    self::register_items( true );

	    $v = Tbp::get_version();
	    themify_enque_script( 'tbp-dynamic-content', TBP_URL . 'admin/js/tbp-dynamic-content.js', $v, array( 'themify-builder-app-js' ) );
	    wp_localize_script( 'tbp-dynamic-content', 'tbpDynamic',
		    array(
			    'items' => self::get_list(),
			    'field_name' => self::FIELD_NAME,
			    'v'=>$v,
			    'd_label'=>__('Dynamic','tbp'),
			    'emptyVal'=>__('Empty Value','tbp'),
			    'placeholder_image' => TBP_URL . 'public/img/transparent.png'
		    )
	    );
	}

	/**
	 * Generate preview value
	 *
	 * Hooked to "wp_ajax_tpb_get_dynamic_content_preview"
	 */
	public static function preview() {
		check_ajax_referer( 'tf_nonce', 'nonce' );
		$options = ! empty( $_POST['values'] )? json_decode( stripslashes_deep( $_POST['values'] ), true ) : array();
		if ( isset( $options['item'] ) ) {
		    Themify_Builder::$frontedit_active = true;
		    // before rendering the dynamic value, first set up the WP Loop
		    Tbp_Utils::$isLoop = true;
		    if ( isset( $_POST['tbp_post_id'] )) {
			$post_id = (int) $_POST['tbp_post_id'];
			if ( ! empty( $_POST['type'] ) && taxonomy_exists( $_POST['type'] ) && term_exists( $post_id, $_POST['type'] ) ) {
				$term = get_term( $post_id, '', ARRAY_A );
				/* setup taxonomy archive loop */
				$tax_query = new WP_Query([
					'tax_query' => [
						[
							'taxonomy' => $term['taxonomy'],
							'field' => 'term_id',
							'terms' => $term['term_id'],
						]
					],
					'posts_per_page' => 1,
				]);
				global $wp_query;
				$wp_query = $tax_query;
				the_post();
			}
			elseif ( $post_object = get_post( $post_id ) ) {
			    setup_postdata( $GLOBALS['post'] =& $post_object );
			}
		    }
		    self::register_items();
		    /* setup the environment for ACF Repeater field preview */
		    if ( isset( $options['key'] ) && substr( $options['key'], 0, 9 ) === 'repeater:'&& substr( $options['item'], 0, 3 ) === 'ACF' ) {
			$pieces = explode( ':', $options['key'] );
			if ( have_rows( $pieces[1] ) ) {
			    the_row();
			}
		    }
		    $value = array( 'value' => self::get_value( $options ) );
		} 
		else {
		    $value = array( 'error' => __( 'Invalid value.', 'tbp' ) );
		}
		die( json_encode( $value ) );
	}
	
	public static function options() {
		check_ajax_referer( 'tf_nonce', 'nonce' );

		$items = self::register_items();

		$items_list = $items_settings = array();
		$categories = array(
			'disabled' => '',
			'general' => __( 'General', 'tbp' ),
			'post' => __( 'Post', 'tbp' ),
			'wc' => __( 'WooCommerce', 'tbp' ),
			'advanced' => __( 'Advanced', 'tbp' ),
			'ptb' => __( 'Themify Post Type Builder', 'tbp' ),
			'acf' => __( 'Advanced Custom Fields', 'tbp' ),
		);
		$items_list['empty'] = array( 'options' => array( '' => '' ) );
		foreach ( $items as $id => $class ) {
			$cat_id = $class->get_category();
			if ( ! isset( $items_list[ $cat_id ] ) ) {
			    $items_list[ $cat_id ] = array(
					'label' => $categories[ $cat_id ],
					'options' => array()
			    );
			}
			$items_list[ $cat_id ]['options'][ $id ] = $class->get_label();

			if ( $options = $class->get_options() ) {
				$items_settings[ $id ] = array(
					'type' => 'group',
					'options' =>  $options,
					'wrap_class' => 'field_' . $id,
				);
			}
		}
		$data = array();
		foreach($categories as $k=>$v){
			if(isset($items_list[$k])){
				$data[$k]=$items_list[$k];
			}
		}
		$items=$categories=$items_list=null;
		/* fields specific to Advanced Custom Fields plugin */
		$items_settings['general_acf'] = array(
			'type' => 'group',
			'options' => self::get_acf_ctx_fields(),
			'wrap_class' => 'tbp_dynamic_content_acf_ctx',
		);
		$items_settings['general_text'] = array(
			'type' => 'group',
			'options' => array(
				array(
					'label' => __( 'Text Before', 'tbp' ),
					'id' => 'text_before',
					'type' => 'text'
				),
				array(
					'label' => __( 'Text After', 'tbp' ),
					'id' => 'text_after',
					'type' => 'text'
				),
			),
			'wrap_class' => 'field_general_text field_general_textarea field_general_wp_editor'
		);
		$items_settings['general_url'] = array(
			'type' => 'group',
			'options' => array(
				array(
					'label' => __( 'URI Scheme', 'tbp' ),
					'id' => 'uri_scheme',
					'type' => 'select',
					'options' => array(
						'0' => __( 'None', 'tbp' ),
						'tel' => __( 'Telephone', 'tbp' ),
						'mailto' => __( 'Email', 'tbp' ),
						'sms' => __( 'Text Message', 'tbp' ),
						'fax' => __( 'Fax', 'tbp' ),
					)
				),
			),
			'wrap_class' => 'field_general_url',
		);
		$items_settings['general_condition'] = array(
			'type' => 'group',
			'options' => array(
				array(
					'type' => 'separator',
					'label' => __('Display Condition', 'tbp')
				),
				array(
					'label' => __( 'When Empty:', 'tbp' ),
					'id' => 'condition',
					'type' => 'select',
					'options' => array(
						'' => '',
						'hide_module' => __( 'Hide the module', 'tbp' ),
						'hide_subrow' => __( 'Hide the subrow', 'tbp' ),
						'hide_row' => __( 'Hide the row', 'tbp' ),
						'show_text' => __( 'Show Fallback Value', 'tbp' ),
					)
				),
				array(
					'label' => __( 'Fallback Value', 'tbp' ),
					'id' => '__fb_val',
					'type' => 'text',
					'help' => __( 'Set a default value to show when the dynamic item is empty.', 'tbp' ),
				),
			),
			'wrap_class' => 'tbp_dynamic_content_condition field_general_text field_general_textarea field_general_wp_editor field_general_url field_general_image field_general_video field_general_audio',
		);
		$options = array(
			array(
				'id' => 'item',
				'type' => 'select',
				'options' => $data,
				'control' => false,
				'optgroup' => true
			),
			array(
				'type' => 'group',
				'options' => $items_settings,
				'wrap_class' => 'field_settings'
			),
		);
		die( json_encode( $options ) );
	}

	/**
	 * Returns Context options used for ACF plugin
	 *
	 * @return array
	 */
	public static function get_acf_ctx_fields() {
		return array(
			array(
				'label' => __( 'Context', 'tbp' ),
				'id' => 'acf_ctx',
				'type' => 'select',
				'options' => array(
					'' => __( 'Current post', 'tbp' ),
					'term' => __( 'Taxonomy terms', 'tbp' ),
					'user' => __( 'Current logged-in user', 'tbp' ),
					'author' => __( 'Author of current post', 'tbp' ),
					'option' => __( 'Option', 'tbp' ),
					'custom' => __( 'Custom', 'tbp' ),
				),
				'binding' =>array(
					'empty' => array( 'hide' => array( 'acf_ctx_c' ) ),
					'not_empty' => array( 'hide' => array( 'acf_ctx_c' ) ),
					'custom' => array( 'show' => array( 'acf_ctx_c' ) )
				)
			),
			array(
				'label' => __( 'Custom Context', 'tbp' ),
				'id' => 'acf_ctx_c',
				'type' => 'text',
				'help' => sprintf( __( 'Custom post ID, or an object ID as expected by ACF (<a href="%s" target="_blank">ref</a>).', 'tbp' ), 'https://www.advancedcustomfields.com/resources/get_field/' ),
			),
		);
	}
}

class Tbp_Dynamic_Item {

	/**
	 * Returns true if this item is available.
	 *
	 * @return bool
	 */
	public function is_available() {
		return true;
	}
	/**
	 * Returns an array of Builder field types this item applies to.
	 *
	 * @return array
	 */
	public function get_type() {
		return array();
	}

	/**
	 * Returns the category this item belongs to
	 *
	 * @return string
	 */
	public function get_category() {
		return '';
	}

	public function get_label() {
		return '';
	}

	public function get_value( $args = array() ) {
		return null;
	}

	public function get_options() {
		return array();
	}
}