<?php
/**
 * @package    Themify Builder Pro
 * @link       https://themify.me/
 */
class Tbp_Dynamic_Item_EventPostDate extends Tbp_Dynamic_Item {

	function is_available() {
		return function_exists( 'themify_event_post_date' );
	}

	function get_category() {
		return 'post';
	}

	function get_type() {
		return array( 'text', 'textarea', 'wp_editor' );
	}

	function get_label() {
		return __( 'Themify Event Post - Event Date', 'tbp' );
	}

	function get_value( $args = array() ) {
		$value = '';
		if ( ! isset( $args['date_format'] ) || $args['date_format'] === 'default' ) {
		    $date_format = get_option( 'date_format' );
	    } elseif ( $args['date_format'] === 'custom' ) {
		    $date_format = isset( $args['custom_date_format'] ) ? $args['custom_date_format'] : '';
	    } else {
		    $date_format = $args['date_format'];
	    }
		if ( ! isset( $args['time_format'] ) || $args['time_format'] === 'default' ) {
		    $time_format = get_option( 'time_format' );
	    } elseif ( $args['time_format'] === 'custom' ) {
		    $time_format = isset( $args['custom_time_format'] ) ? $args['custom_time_format'] : '';
	    } else {
		    $time_format = $args['time_format'];
	    }

		$the_query = Tbp_Utils::get_actual_query();
		if ( $the_query === null || $the_query->have_posts() ) {
			if ( $the_query !== null ) {
				$the_query->the_post();
			}

			ob_start();
			themify_event_post_date( $date_format, $time_format );
			$value = ob_get_clean();
		}
		if ( $the_query !== null ) {
			wp_reset_postdata();
		}

		return $value;
	}

	function get_options() {
		$custom_format_help = sprintf( __( 'For information on how to format date and time see <a href="%s" target="_blank">Codex</a>.', 'tbp' ), 'https://wordpress.org/support/article/formatting-date-and-time/' );

		return array(
			array(
				'label' => __( 'Date Format', 'tbp' ),
				'id' => 'date_format',
				'type' => 'select',
				'options' => array(
					'default' => __( 'Default', 'tbp' ),
					'F j, Y' => date_i18n( 'F j, Y' ),
					'Y-m-d'  => date_i18n( 'Y-m-d' ),
					'm/d/Y'  => date_i18n( 'm/d/Y' ),
					'd/m/Y'  => date_i18n( 'd/m/Y' ),
					'custom' => __( 'Custom', 'tbp' ),
				),
				'binding' =>array(
				  'not_empty'=>array('hide'=>array('custom_date_format')),
				  'custom'=>array('show'=>array('custom_date_format'))
				)
			),
			array(
				'label' => __( 'Custom Date Format', 'tbp' ),
				'id' => 'custom_date_format',
				'type' => 'text',
				'help' => $custom_format_help,
			),
			array(
				'label' => __( 'Time Format', 'tbp' ),
				'id' => 'time_format',
				'type' => 'select',
				'options' => array(
					'default' => __( 'Default', 'tbp' ),
					'g:i a' => date_i18n( 'g:i a' ),
					'g:i A'  => date_i18n( 'g:i A' ),
					'H:i'  => date_i18n( 'H:i' ),
					'custom' => __( 'Custom', 'tbp' ),
				),
				'binding' =>array(
				  'not_empty'=>array('hide'=>array('custom_time_format')),
				  'custom'=>array('show'=>array('custom_time_format'))
				)
			),
			array(
				'label' => __( 'Custom Time Format', 'tbp' ),
				'id' => 'custom_time_format',
				'type' => 'text',
				'help' => $custom_format_help,
			),
		);
	}
}
