<?php
/**
 * @package    Themify Builder Pro
 * @link       https://themify.me/
 */
class Tbp_Dynamic_Item_PTBNumber extends Tbp_Dynamic_Item {

	function is_available() {
		return function_exists( 'run_ptb' );
	}

	function get_category() {
		return 'ptb';
	}

	function get_type() {
		return array( 'text', 'textarea', 'wp_editor','slider_range' );
	}

	function get_label() {
		return __( 'PTB (Number)', 'tbp' );
	}

	function get_value( $args = array() ) {
		$value='';
		if(!empty($args['field'])){
		    $the_query = Tbp_Utils::get_actual_query();
		    if($the_query===null || $the_query->have_posts()){
			if($the_query!==null){
			    $the_query->the_post();
			}

				$field_name = explode( ':', $args['field'] );
				$value = get_post_meta(get_the_ID(), "ptb_{$field_name[1]}", true );

				$display = isset( $args['display'] ) ? $args['display'] : 'from';
				$decimals = isset( $args['decimals'] ) ? (int) $args['decimals'] : 2;
				$dec_point = isset( $args['dec_point'] ) ? $args['dec_point'] : '.';
				$thousands_sep = isset( $args['thousands_sep'] ) ? $args['thousands_sep'] : ',';
				if ( is_array( $value ) ) { // range input
					$value = $value[ $display ];
				}
				$value = number_format( floatval( $value ), $decimals, $dec_point, $thousands_sep );

		    }
		    if($the_query!==null){
			wp_reset_postdata();
		    }
		}
		return $value;
	}

	function get_options() {
		return array(
			array(
				'label' => __( 'Field', 'tbp' ),
				'id' => 'field',
				'type' => 'select',
				'options' => Tbp_Utils::get_ptb_fields_by_type( 'number' ),
			),
			array(
				'label' => __( 'Display', 'tbp' ),
				'id' => 'display',
				'type' => 'select',
				'options' => array(
					'from' => __( 'From', 'tbp' ),
					'to' => __( 'To', 'tbp' ),
				),
				'help' => __( 'In Range inputs, select to display the From value or To value.', 'tbp' ),
			),
			array(
				'label' => __( 'Decimals', 'tbp' ),
				'id' => 'decimals',
				'type' => 'text',
			),
			array(
				'label' => __( 'Decimal Point', 'tbp' ),
				'id' => 'dec_point',
				'type' => 'text',
			),
			array(
				'label' => __( 'Thousands Separator', 'tbp' ),
				'id' => 'thousands_sep',
				'type' => 'text',
			),
		);
	}
}