<?php
/**
 * @package    Themify Builder Pro
 * @link       https://themify.me/
 */
class Tbp_Dynamic_Item_PTBRating extends Tbp_Dynamic_Item {

	function is_available() {
		return function_exists( 'run_ptb' );
	}

	function get_category() {
		return 'ptb';
	}

	function get_type() {
		return array( 'wp_editor' );
	}

	function get_label() {
		return __( 'PTB (Rating)', 'tbp' );
	}

	function get_value( $args = array() ) {
		if ( empty( $args['field'] ) || ! function_exists( 'ptb_get_field' ) ) {
			return;
		}

		list( $post_type, $field_name ) = explode( ':', $args['field'] );

		/* the option names translate directly to PTB's option names for the field */
		$args = wp_parse_args( $args, array(
			'icon' => 'fa-star',
			'size' => 'small',
			'vcolor' => 'rgba(250, 225, 80, 1)',
		) );
		return ptb_get_field( $field_name, $args );
	}

	function get_options() {
		return array(
			array(
				'label' => __( 'Field', 'tbp' ),
				'id' => 'field',
				'type' => 'select',
				'options' => Tbp_Utils::get_ptb_fields_by_type( 'rating' ),
			),
			array(
				'label' => __( 'Size', 'tbp' ),
				'id' => 'size',
				'type' => 'select',
				'options' => array(
					'small' => __( 'Small', 'tbp' ),
					'medium' => __( 'Medium', 'tbp' ),
					'large' => __( 'Large', 'tbp' ),
				),
			),
			array(
				'id' => 'icon',
				'type' => 'icon',
				'label' => __( 'Icon', 'tbp' ),
			),
			array(
				'id' => 'vcolor',
				'type' => 'color',
				'label' => __( 'Color', 'tbp' ),
			),
		);
	}
}