<?php
/**
 * @package    Themify Builder Pro
 * @link       https://themify.me/
 */
class Tbp_Dynamic_Item_PTBRatingAsText extends Tbp_Dynamic_Item {

	function is_available() {
		return function_exists( 'run_ptb' );
	}

	function get_category() {
		return 'ptb';
	}

	function get_type() {
		return array( 'text', 'textarea', 'slider_range' );
	}

	function get_label() {
		return __( 'PTB (Rating)', 'tbp' );
	}

	function get_value( $args = array() ) {
		$value = '';
		if ( ! empty( $args['field'] ) ) {
			$show = isset( $args['show'] ) ? $args['show'] : 'total';
			list( $post_type, $field_name ) = explode( ':', $args['field'] );
			$cf_value = get_post_meta( get_the_ID(), "ptb_{$field_name}", true );
			if ( is_string( $cf_value ) ) {
				$value = $cf_value;
			} else if ( is_array( $cf_value ) && isset( $cf_value[ $show ] ) ) {
				if ( $show === 'total' ) {
					$value = $cf_value['count'] > 0 ? floatval( $cf_value['total'] / $cf_value['count'] ) : 0;
				} else {
					$value = $cf_value[ $show ];
				}
			}
		}

		return $value;
	}

	function get_options() {
		return array(
			array(
				'label' => __( 'Field', 'tbp' ),
				'id' => 'field',
				'type' => 'select',
				'options' => Tbp_Utils::get_ptb_fields_by_type( 'rating' ),
			),
			array(
				'label' => __( 'Show', 'tbp' ),
				'id' => 'show',
				'type' => 'select',
				'options' => array(
					'total' => __( 'Vote Result', 'tbp' ),
					'count' => __( 'Vote Counts', 'tbp' ),
				),
			),
		);
	}
}