<?php
/**
 * @package    Themify Builder Pro
 * @link       https://themify.me/
 */
class Tbp_Dynamic_Item_PTBRepeatableText extends Tbp_Dynamic_Item {

	function is_available() {
		return function_exists( 'run_ptb' );
	}

	function get_category() {
		return 'ptb';
	}

	function get_type() {
		return array( 'wp_editor' );
	}

	function get_label() {
		return __( 'PTB (Repeatable Text)', 'tbp' );
	}

	function get_value( $args = array() ) {

		if ( empty( $args['field'] ) ) {
			return;
		}
		
		$args = wp_parse_args( $args, array(
			'icon' => 'fas fa-circle',
			'color' => '',
		) );
		$output = '';
		list( $post_type, $field_name ) = explode( ':', $args['field'] );
		$value = get_post_meta( get_the_ID(), "ptb_{$field_name}", true );
		$value = array_filter( $value );
		if ( is_array( $value ) && ! empty( $value ) ) {
			// repeatable Text fields
			$output = '<div class="module-icon icon_vertical">';
			foreach ( $value as $icon_text ) {
				$color = ! empty( $args['color'] ) ? ' style="color: ' . $args['color'] . '"' : '';
				$output .= '<div class="module-icon-item">';
					$output .=  '<span class="tbp_icon"' . $color . '>' . themify_get_icon( $args['icon'] ) . '</span> ';
					$output .= $icon_text;
				$output .= '</div>';
			}
			$output .= '</div>';
		}

		return $output;
	}

	function get_options() {
		$options = array();

		/* collect "text" field types in all post types */
		$post_types = Themify_Builder_Model::get_public_post_types();
		foreach ( $post_types as $post_type => $post_type_label ) {
			$ptb_fields = PTB::$options->get_cpt_cmb_options( $post_type );
			if ( ! empty( $ptb_fields ) ) {
				foreach ( $ptb_fields as $key => $field ) {
					if ( $field['type'] === 'text' && ! empty( $field['repeatable'] ) ) {
						$name = PTB_Utils::get_label( $field['name'] );
						$options[ "{$post_type}:{$key}" ] = sprintf( '%s: %s', $post_type_label, $name );
					}
				}
			}
		}

		return array(
			array(
				'label' => __( 'Field', 'tbp' ),
				'id' => 'field',
				'type' => 'select',
				'options' => $options,
			),
			array(
				'label' => __( 'Icon', 'tbp' ),
				'id' => 'icon',
				'type' => 'icon',
			),
			array(
				'id' => 'color',
				'type' => 'color',
				'label' => __( 'Icon Color', 'tbp' ),
			),
		);
	}
}
