<?php
/**
 * Builder Plugin Compatibility Code
 *
 * @package    Themify_Builder Pro
 */
/**
 * @link https://www.advancedcustomfields.com/
 */
class Themify_Builder_Plugin_Compat_acfPro {

	static function init() {
		if ( ! class_exists( 'Themify_Builder_Plugin_Compat_acf' ) ) {
			/* load ACF compatibility file as well */
			include dirname( __FILE__ ) . '/acf.php';
			Themify_Builder_Plugin_Compat_acf::init();
		}

		add_filter( 'tbp_dynamic_query_items', array( __CLASS__, 'tbp_dynamic_query_items' ) );
	}

	/**
	 * Add ACF pro's "relational" fields as an option to Dyanmic Query
	 *
	 * @return array
	 */
	public static function tbp_dynamic_query_items( $items ) {
		$items['acf'] = 'Tbp_Dynamic_Query_acf';

		return $items;
	}
}

class Tbp_Dynamic_Query_acf {

	static function get_id() {
		return 'acf';
	}

	static function get_label() {
		return __( 'ACF Pro Relational', 'tbp' );
	}

	static function get_options() {
		return array(
			array(
				'id' => 'acf_field',
				'type' => 'select',
				'label' => __( 'ACF Relational Field', 'tbp' ),
				'options' => Tbp_Utils::get_acf_fields_by_type( [ 'relationship', 'post_object', 'taxonomy' ] ),
			),
			array(
				'id' => 'acf_ctx',
				'type' => 'select',
				'label' => __( 'ACF Context', 'tbp' ),
				'options' => array(
					'' => __( 'Current post', 'tbp' ),
					'term' => __( 'Taxonomy terms', 'tbp' ),
					'user' => __( 'Current logged-in user', 'tbp' ),
					'author' => __( 'Author of current post', 'tbp' ),
					'option' => __( 'Option', 'tbp' ),
				),
			),
		);
	}

	static function pre_get_posts( &$query, $settings ) {
		if ( empty( $settings['acf_field'] ) ) {
			return;
		}

		list( $group_id, $field_id ) = explode( ':', $settings['acf_field'] );
		$value = get_field( $field_id, Tbp_Utils::acf_get_context( $settings ) );
		$field_object = get_field_object( $field_id );
		if ( ! empty( $value ) && ! empty( $field_object ) ) {
			/* Taxonomy field type in ACF */
			if ( $field_object['type'] === 'taxonomy' ) {
				if ( $field_object['return_format'] === 'object' ) {
					$value = wp_list_pluck( $value, 'term_id' );
				}
				/* get the ID of the posts assigned to the selected taxonomy term */
				$value = get_posts( [
					'fields' => 'ids',
					'post_type' => 'any',
					'posts_per_page' => $query->get( 'posts_per_page' ),
					'tax_query' => [
						[
							'taxonomy' => $field_object['taxonomy'],
							'field' => 'term_id',
							'terms' => $value,
						]
					]
				] );
			} else { /* Relationship and Post Object field types in ACF */
				if ( $field_object['type'] === 'post_object' && $field_object['multiple'] !== 1 ) {
					$value = [ $value ];
				}
				if ( $field_object['return_format'] === 'object' ) {
					$value = wp_list_pluck( $value, 'ID' );
				}
				$query->set( 'posts_per_page', -1 );
			}

			/* disable any post from showing in the module */
			if ( empty( $value ) ) {
				return false;
			}

			$query->set( 'post_type', 'any' );
			$query->set( 'post__in', $value );

			return true;
		}
	}
}