<?php
/**
 * Builder Plugin Compatibility Code
 *
 * @package    Themify_Builder Pro
 */
/**
 * @link https://themify.me/ptb-addons/submissions
 */
class Themify_Builder_Plugin_Compat_ptbSubmission {

	static function init() {
		add_filter( 'tbp_dynamic_query_items', array( __CLASS__, 'tbp_dynamic_query_items' ) );
	}

	/**
	 * Add PTB Relation as an option to Dyanmic Query
	 *
	 * @return array
	 */
	public static function tbp_dynamic_query_items( $items ) {
		$items['ptbSubmission'] = 'Tbp_Dynamic_Query_ptbSubmission';

		return $items;
	}
}

class Tbp_Dynamic_Query_ptbSubmission {

	static function get_id() {
		return 'ptbSubmission';
	}

	static function get_label() {
		return __( 'PTB Submissions by current user', 'tbp' );
	}

	static function get_options() {
		return array(
            array(
				'id' => 'ptbSubmission_post_type',
				'type' => 'select',
				'label' => __( 'Post Type', 'tbp' ),
				'options' => Themify_Builder_Model::get_public_post_types(),
            ),
            array(
				'id' => 'ptbSubmission_show',
				'type' => 'select',
				'label' => __( 'Show', 'tbp' ),
				'options' => [
					'publish' => __( 'Approved Posts', 'tbp' ),
					'pending' => __( 'Pending Approval', 'tbp' ),
					'both' => __( 'All', 'tbp' ),
				],
            ),
		);
	}

	static function pre_get_posts( &$query, $settings ) {
		if ( ! is_user_logged_in() ) {
			return false;
		}

		$query->set( 'post_type', $settings['ptbSubmission_post_type'] );
		$query->set( 'author', get_current_user_id() );
		$post_status = $settings['ptbSubmission_show'] === 'both' ? [ 'publish', 'pending' ] : $settings['ptbSubmission_show'];
		$query->set( 'post_status', $post_status );

		return true;
	}
}