<?php
if (!defined('ABSPATH'))
	exit; // Exit if accessed directly

/**
 * Module Name: PTB Repeater
 * Description: 
 */

class TB_PTB_Repeater_Module extends Themify_Builder_Component_Module {

	function __construct() {
		if ( method_exists( 'Themify_Builder_Model', 'add_module' ) ) {
            parent::__construct( 'ptb-repeater' );
        } else {
			parent::__construct(array(
				'name' => __('PTB Repeater', 'tbp'),
				'slug' => 'ptb-repeater',
				'category' => array( 'general' )
			));
		}
	}

    public function get_name(){
		return __( 'PTB Repeater', 'tbp' );
    }

    public function get_group() {
		return array( 'general' );
    }

	public function get_assets() {
		return array();
	}

	public function get_icon(){
		return 'loop';
	}
	
	public function get_options() {
		return array(
			array(
				'id'      => 'builder_content',
				'type'    => 'tbp_advanched_layout',
				'control' => false
			),
			array(
				'id' => 'key',
				'type' => 'select',
				'label' => __( 'Field Name', 'tbp' ),
				'wrap_class' => 'tb_disable_dc',
				'options' => array_merge( [ '' => '' ], self::get_repeatable_fields() ),
				'control' => false,
				'required' => [
					'message' => __( 'You have to select a repeatable field.', 'tbp' ),
					'rule' => 'not_empty',
				]
			),
			array(
				'id' => 'display',
				'type' => 'radio',
				'label' => __('Display as', 'tbp'),
				'options' => array(
				    array('value' => 'grid','name'=>__('Grid', 'tbp')),
				    array('value' => 'slider','name'=>__('Slider', 'tbp'))
				),
				'option_js' => true
			),
			array(
				'type' => 'group',
				'options' => array(
					array(
						'id' => 'slider',
						'type' => 'slider',
						'label' => __('Slider Options', 'tbp'),
						'slider_options' => true,
					),
				),
				'wrap_class' => 'tb_group_element_slider'
			),
			array(
				'id' => 'grid_layout',
				'type' => 'layout',
				'label' => __( 'Layout', 'tbp' ),
				'mode' => 'sprite',
				'control' => array(
				    'classSelector' => '.builder-posts-wrap'
				),
				'options' => array(
					array('img' => 'list_post', 'value' => 'list-post', 'label' => __('Grid1', 'tbp')),
					array('img' => 'grid2', 'value' => 'grid2', 'label' => __('Grid 2', 'tbp')),
					array('img' => 'grid3', 'value' => 'grid3', 'label' => __('Grid 3', 'tbp')),
					array('img' => 'grid4', 'value' => 'grid4', 'label' => __('Grid 4', 'tbp')),
					array('img' => 'grid5', 'value' => 'grid5', 'label' => __('Grid 5', 'tbp')),
					array('img' => 'grid6', 'value' => 'grid6', 'label' => __('Grid 6', 'tbp')),
				),
				'wrap_class' => 'tb_group_element_grid',
			),
			array( 'type' => 'custom_css_id', 'custom_css' => 'css' ),
		);
	}

	public static function get_repeatable_fields() {
		$type = [ 'video', 'audio', 'file', 'gallery', 'slider', 'accordion' ];
		$options = array();

		$ptb = PTB::$options->get_custom_post_types();
		foreach ( $ptb as $post_type_key => $post_type ) {
			if ( is_array( $post_type->meta_boxes ) ) {
				foreach ( $post_type->meta_boxes as $key => $field ) {
					if ( in_array( $field['type'], $type, true ) ) {
					    $label = PTB_Utils::get_label( $post_type->plural_label );
					    $name = PTB_Utils::get_label( $field['name'] );
						$options[ "{$post_type_key}:{$key}:{$field['type']}" ] = sprintf( '%s: %s', $label, $name );
					}
				}
			}
		}

		return $options;
	}

	public function get_styling() {
		$general = array(
			// Background
			self::get_expand('bg', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_image('', 'b_c_g', 'bg_c', 'background-color')
					)
					),
					'h' => array(
					'options' => array(
						self::get_image('', 'b_c_g', 'bg_c', 'background-color', 'h')
					)
					)
				))
			)),
			// Font
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_font_family('', 'f_f_g'),
						self::get_color_type( ['> .tbp_advanced_archive_wrap'] ),
						self::get_font_size('', 'f_s_g'),
						self::get_line_height('', 'l_h_g'),
						self::get_letter_spacing('', 'l_s_g'),
						self::get_text_align( ['', ' .post'], 't_a_g'),
						self::get_text_transform('', 't_t_g'),
						self::get_font_style('', 'f_g', 'f_b'),
						self::get_text_shadow('', 't_sh'),
					)
					),
					'h' => array(
					'options' => array(
						self::get_font_family('', 'f_f_g', 'h'),
						self::get_color_type( ['> .tbp_advanced_archive_wrap'], 'h'),
						self::get_font_size('', 'f_s_g', '', 'h'),
						self::get_font_style('', 'f_g', 'f_b', 'h'),
						self::get_text_shadow('','t_sh','h'),
					)
					)
				))
			)),
			// Padding
			self::get_expand('p', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_padding('', 'g_p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_padding('', 'g_p', 'h')
					)
					)
				))
			)),
			// Margin
			self::get_expand('m', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_margin('', 'g_m')
					)
					),
					'h' => array(
					'options' => array(
						self::get_margin('', 'g_m', 'h')
					)
					)
				)),
			)),
			// Border
			self::get_expand('b', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_border('', 'g_b')
					)
					),
					'h' => array(
					'options' => array(
						self::get_border('', 'g_b', 'h')
					)
					)
				))
			)),
			// Filter
			self::get_expand('f_l',
				array(
					self::get_tab(array(
						'n' => array(
							'options' => count($a = self::get_blend())>2 ? array($a) : $a
						),
						'h' => array(
							'options' => count($a = self::get_blend('','bl_m_h','h'))>2 ? array($a + array('ishover'=>true)) : $a
						)
					))
				)
			),
			// Width
			self::get_expand('w', array(
				self::get_width('', 'w')
			)),
			// Height & Min Height
			! method_exists( $this, 'get_max_height' ) ? array() :
			self::get_expand('ht', array(
					self::get_height(),
					self::get_min_height(),
					self::get_max_height()
				)
			),
			// Rounded Corners
			self::get_expand('r_c', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_border_radius('', 'r_c')
						)
					),
					'h' => array(
						'options' => array(
							self::get_border_radius('', 'r_c', 'h')
						)
					)
				))
			)),
			// Shadow
			self::get_expand('sh', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_box_shadow('', 'g_sh')
						)
					),
					'h' => array(
						'options' => array(
							self::get_box_shadow('', 'g_sh', 'h')
						)
					)
				))
			)),
			// Display
			self::get_expand('disp', self::get_display())
		);

		$items = array(
			// Items Background
			self::get_expand('bg', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
					   self::get_color(' .post', 'ptb_rpt_bg', 'bg_c', 'background-color')
					)
					),
					'h' => array(
					'options' => array(
						self::get_color(' .post', 'ptb_rpt_bg', 'bg_c', 'background-color','h')
					)
					)
				))
			)),
			// Items Margin
			self::get_expand('m', array(
			self::get_tab(array(
				'n' => array(
				'options' => array(
					self::get_heading_margin_multi_field(' .post', '', 'top', '', 'ptb_rpt_m'),
					self::get_heading_margin_multi_field(' .post', '', 'bottom', '', 'ptb_rpt_m')
				)
				),
				'h' => array(
				'options' => array(
					self::get_heading_margin_multi_field(':hover > .post', '', 'top', '', 'ptb_rpt_m_h'),
					self::get_heading_margin_multi_field(':hover > .post', '', 'bottom', '', 'ptb_rpt_m_h')
				)
				)
			))
			)),
			// Items Padding
			self::get_expand('p', array(
			self::get_tab(array(
				'n' => array(
					'options' => array(
						self::get_padding(' .post', 'ptb_rpt_p'),
					)
				),
				'h' => array(
					'options' => array(
						self::get_padding(' .post', 'ptb_rpt_p_h'),
					)
				)
			))
			)),
			// Items Border
			self::get_expand('b', array(
			self::get_tab(array(
				'n' => array(
					'options' => array(
						self::get_border(' .post', 'ptb_rpt_b'),
					)
				),
				'h' => array(
					'options' => array(
						self::get_border(' .post', 'ptb_rpt_b_h'),
					)
				)
			))
			)),
			// Items Border Radius
			self::get_expand('r_c', array(
			self::get_tab(array(
				'n' => array(
					'options' => array(
						self::get_border_radius(' .post', 'ptb_rpt_rc'),
					)
				),
				'h' => array(
					'options' => array(
						self::get_border_radius(' .post', 'ptb_rpt_rc_h'),
					)
				)
			))
			)),
			// Items Shadow
			self::get_expand('sh', array(
			self::get_tab(array(
				'n' => array(
					'options' => array(
						self::get_box_shadow(' .post', 'ptb_rpt_sh'),
					)
				),
				'h' => array(
					'options' => array(
						self::get_box_shadow(' .post', 'ptb_rpt_sh_h'),
					)
				)
			))
			)),
		);

		return array(
			'type' => 'tabs',
			'options' => array(
				'g' => array(
					'options' => $general
				),
				'items' => array(
					'label' => __('Items', 'ptb_rt'),
					'options' => $items
				)
			)
		);
	}

	public function get_animation() {
		return false;
	}

	public function get_category() {
	    return array( 'general' );
	}
}

if ( class_exists( 'PTB_CMB_Base' ) ) {
    if ( method_exists( 'Themify_Builder_Model', 'add_module' ) ) {
        new TB_PTB_Repeater_Module();
    } else {
        Themify_Builder_Model::register_module('TB_PTB_Repeater_Module');
    }
}