<?php
if ( ! defined( 'ABSPATH' ) )
	exit; // Exit if accessed directly

/**
 * Module Name: Read Time
 * Description: Provides estimates on how long it takes to read the post.
 */
class TB_Read_Time_Module extends Themify_Builder_Component_Module {

	function __construct() {
		if ( method_exists( 'Themify_Builder_Model', 'add_module' ) ) {
			parent::__construct( 'readtime' );
		}
		else{//backward
			parent::__construct(array(
				'name' => __( 'Reading Time', 'tbp' ),
				'slug' => 'readtime',
				'category' => array( 'single' )
			));
		}
		add_action( 'wp_ajax_tbp_readtime_calculate', [ __CLASS__, 'wp_ajax_tbp_readtime_calculate' ] );
		add_action( 'themify_builder_save_data', [ __CLASS__, 'clear_cache' ] );
	}

    public function get_assets() {
		return array(
			'ver' => Tbp::get_version(),
			'js' => TBP_JS_MODULES . 'readtime',
		);
    }

    public function get_name(){
        return __( 'Reading Time', 'tbp' );
    }

	public function get_icon(){
		return 'time';
	}

	public function get_group() {
		return array( 'single' );
	}

	public function get_options() {
		return [
			array(
				'id' => 'm_t',
				'type' => 'title'
			),
			array(
				'id' => 'ic',
				'type' => 'icon',
				'label' => 'icon'
			),
            array(
                'id' => 'b_text',
                'type' => 'text',
                'label' => __('Before Text', 'tbp'),
                'control' => array(
                    'selector' => '.tbp_rd_tm_b'
                )
            ),
            array(
                'id' => 'a_text',
                'type' => 'text',
                'label' => __('After Text', 'tbp'),
                'control' => array(
                    'selector' => '.tbp_rd_tm_a'
                )
            ),
            array(
                'id' => 'less',
                'type' => 'text',
                'label' => __('Less than 1 minute', 'tbp'),
                'control' => array(
                    'selector' => '.tbp_rd_tm_less'
                )
            ),
		];
	}

	public function get_styling() {
		$general = array(
			// Background
			self::get_expand('bg', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_image('', 'b_i','bg_c','b_r','b_p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_image('', 'b_i','bg_c','b_r','b_p', 'h')
					)
					)
				))
			)),
			// Font
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_font_family('', 'f_f'),
						self::get_color('', 'f_c'),
						self::get_font_size('', 'f_s'),
						self::get_line_height('', 'l_h'),
						self::get_letter_spacing('', 'l_s'),
						self::get_text_align('', 't_a'),
						self::get_text_transform('', 't_t'),
						self::get_font_style('', 'f_st', 'f_w'),
						self::get_text_decoration('', 't_d_r'),
						self::get_text_shadow('','t_sh'),
					)
					),
					'h' => array(
					'options' => array(
						self::get_font_family('', 'f_f_h'),
						self::get_color('', 'f_c_h',null, null, 'h'),
						self::get_font_size('', 'f_s', '', 'h'),
						self::get_font_style('', 'f_st', 'f_w', 'h'),
						self::get_text_decoration('', 't_d_r', 'h'),
						self::get_text_shadow('','t_sh','h'),
					)
					)
				))
			)),
			// Padding
			self::get_expand('p', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_padding('', 'p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_padding('', 'p', 'h')
					)
					)
				))
			)),
			// Margin
			self::get_expand('m', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_margin('', 'm')
					)
					),
					'h' => array(
					'options' => array(
						self::get_margin('', 'm', 'h')
					)
					)
				))
			)),
			// Border
			self::get_expand('b', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_border('', 'b')
					)
					),
					'h' => array(
					'options' => array(
						self::get_border('', 'b', 'h')
					)
					)
				))
			)),
			// Filter
			self::get_expand('f_l',
				array(
					self::get_tab(array(
						'n' => array(
							'options' => count($a = self::get_blend())>2 ? array($a) : $a
						),
						'h' => array(
							'options' => count($a = self::get_blend('','bl_m_h','h'))>2 ? array($a + array('ishover'=>true)) : $a
						)
					))
				)
			),
			// Width
			self::get_expand('w', array(
				self::get_width('', 'w')
			)),
			! method_exists( $this, 'get_max_height' ) ? array() :
			// Height & Min Height
			self::get_expand('ht', array(
				self::get_height(),
				self::get_min_height(),
				self::get_max_height()
			)),
			// Rounded Corners
			self::get_expand('r_c', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_border_radius('', 'r_c')
						)
					),
					'h' => array(
						'options' => array(
							self::get_border_radius('', 'r_c', 'h')
						)
					)
				))
			)),
			// Shadow
			self::get_expand('sh', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_box_shadow('', 'sh')
						)
					),
					'h' => array(
						'options' => array(
							self::get_box_shadow('', 'sh', 'h')
						)
					)
				))
			)),
			// Position
			self::get_expand('po', array( self::get_css_position())),
			// Display
			self::get_expand('disp', self::get_display())
		);

		$icon = array(
			// Background
			self::get_expand('bg', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_color(' .tbp_rd_tm_ic', 'b_c_i', 'bg_c', 'background-color')
					)
					),
					'h' => array(
					'options' => array(
						self::get_color(' .tbp_rd_tm_ic', 'b_c_i', 'bg_c', 'background-color', 'h')
					)
					)
				))
			)),
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_color(' .tbp_rd_tm_ic', 'f_c_i' ),
					)
					),
					'h' => array(
					'options' => array(
						self::get_color(' .tbp_rd_tm_ic', 'f_c_i', null,null, 'h' ),
					)
					)
				))
			)),
			// Padding
			self::get_expand('p', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_padding(' .tbp_rd_tm_ic', 'i_p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_padding(' .tbp_rd_tm_ic', 'i_p', 'h')
					)
					)
				))
			)),
			// Margin
			self::get_expand('m', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_margin(' .tbp_rd_tm_ic', 'i_m')
					)
					),
					'h' => array(
					'options' => array(
						self::get_margin(' .tbp_rd_tm_ic', 'i_m', 'h')
					)
					)
				))
			)),
			// Border
			self::get_expand('b', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_border(' .tbp_rd_tm_ic', 'i_b')
					)
					),
					'h' => array(
					'options' => array(
						self::get_border(' .tbp_rd_tm_ic', 'i_b', 'h')
					)
					)
				))
			)),
			// Rounded Corners
			self::get_expand('r_c', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_border_radius(' .tbp_rd_tm_ic', 'i_r_c')
					)
					),
					'h' => array(
					'options' => array(
						self::get_border_radius(' .tbp_rd_tm_ic', 'i_r_c', 'h')
					)
					)
				))
			)),
			// Shadow
			self::get_expand('sh', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_box_shadow(' .tbp_rd_tm_ic', 'i_sh')
					)
					),
					'h' => array(
					'options' => array(
						self::get_box_shadow(' .tbp_rd_tm_ic', 'i_sh', 'h')
					)
					)
				))
			)),
		);

		$text_before = array(
			// Font
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_font_family(' .tbp_rd_tm_b', 'f_f_t_b'),
						self::get_color(' .tbp_rd_tm_b', 'f_c_t_b'),
						self::get_font_size(' .tbp_rd_tm_b', 'f_s_t_b'),
						self::get_letter_spacing(' .tbp_rd_tm_b', 'l_s_t_b'),
						self::get_line_height(' .tbp_rd_tm_b', 'l_h_t_b'),
						self::get_font_style(' .tbp_rd_tm_b', 'f_st_t_b', 'f_w_t_b'),
						self::get_text_shadow(' .tbp_rd_tm_b', 't_sh_t_b'),
					)
					),
					'h' => array(
					'options' => array(
						self::get_font_family(' .tbp_rd_tm_b', 'f_f_t_b', 'h'),
						self::get_color(' .tbp_rd_tm_b', 'f_c_t_b', null,null, 'h' ),
						self::get_font_size(' .tbp_rd_tm_b', 'f_s_t_b', '', 'h'),
						self::get_font_style(' .tbp_rd_tm_b', 'f_st_t_b', 'f_w_t_b', 'h'),
						self::get_text_shadow(' .tbp_rd_tm_b', 't_sh_t_b', 'h'),
					)
					)
				))
			)),
		);

		$text_after = array(
			// Font
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_font_family(' .tbp_rd_tm_a', 'f_f_t_a'),
						self::get_color(' .tbp_rd_tm_a', 'f_c_t_a'),
						self::get_font_size(' .tbp_rd_tm_a', 'f_s_t_a'),
						self::get_letter_spacing(' .tbp_rd_tm_a', 'l_s_t_a'),
						self::get_line_height(' .tbp_rd_tm_a', 'l_h_t_a'),
						self::get_font_style(' .tbp_rd_tm_a', 'f_st_t_a', 'f_w_t_a'),
						self::get_text_shadow(' .tbp_rd_tm_a', 't_sh_t_a'),
					)
					),
					'h' => array(
					'options' => array(
						self::get_font_family(' .tbp_rd_tm_a', 'f_f_t_a', 'h'),
						self::get_color(' .tbp_rd_tm_a', 'f_c_t_a', null,null, 'h' ),
						self::get_font_size(' .tbp_rd_tm_a', 'f_s_t_a', '', 'h'),
						self::get_font_style(' .tbp_rd_tm_a', 'f_st_t_a', 'f_w_t_a', 'h'),
						self::get_text_shadow(' .tbp_rd_tm_a', 't_sh_t_a', 'h'),
					)
					)
				))
			)),
		);

		return array(
			'type' => 'tabs',
			'options' => array(
				'g' => array(
					'options' => $general
				),
				'i' => array(
					'options' => $icon,
					'label' =>'Icon',
				),
				'tb' => array(
					'options' => $text_before,
					'label' => __( 'Text Before', 'tbp' ),
				),
				'ta' => array(
					'options' => $text_after,
					'label' => __( 'Text After', 'tbp' ),
				),
			)
		);
	}

	public function get_live_default() {
		return array(
			'ic' => 'ti-time',
			'less' => __( 'Less than 1 minute', 'tbp' ),
			'a_text' => __( ' Minutes', 'tbp' ),
		);
	}

	public function get_category() {
		return array( 'single', 'archive', 'page' );
	}


	/**
	 * Get the raw unprocessed content for a given $post_id
	 *
	 * @return string
	 */
	public static function get_content( $post_id = null ) {
		if ( empty( $post_id ) ) {
			$post_id = get_the_ID();
		}
		$post = get_post( $post_id );
		$content = $post->post_content;
		if ( ! ThemifyBuilder_Data_Manager::has_static_content( $content ) ) {
			/* check if there's Builder data */
			$builder_data = ThemifyBuilder_Data_Manager::get_data( $post->ID );
			if ( ! empty( $builder_data ) ) {
				/* render a text-only version of Builder data and append that to the content */
				$content .= ThemifyBuilder_Data_Manager::_get_all_builder_text_content( $builder_data );
			}
		}

		return $content;
	}

	/**
	 * Count the number of words in $string
	 * This function is localized. For languages that count ‘words’ by the individual character (such as East Asian languages), the function counts the number of individual characters.
	 *
	 * @return int
	 */
	public static function get_words_count( $string ) {
		/*
		 * translators: If your word count is based on single characters (e.g. East Asian characters),
		 * enter 'characters_excluding_spaces' or 'characters_including_spaces'. Otherwise, enter 'words'.
		 * Do not translate into your own language.
		 */
		if ( strpos( _x( 'words', 'Word count type. Do not translate!' ), 'characters' ) === 0 && preg_match( '/^utf\-?8$/i', get_option( 'blog_charset' ) ) ) {
			$string = trim( preg_replace( "/[\n\r\t ]+/", ' ', $string ), ' ' );
			preg_match_all( '/./u', $string, $words_array );
			$words_array = $words_array[0];
		} else {
			$words_array = preg_split( "/[\n\r\t ]+/", $string, -1, PREG_SPLIT_NO_EMPTY );
		}

		return count( $words_array );
	}

	public function get_plain_content($module) {
		return '';
	}

	public static function wp_ajax_tbp_readtime_calculate() {
		check_ajax_referer( 'tbp_readtime', 'nonce' );

		$ids = json_decode( stripcslashes( $_POST['ids'] ), true );
		$query = new WP_Query( [
			'post_type' => 'any',
			'post_status' => 'any',
			'ignore_sticky_posts' => true,
			'post__in' => $ids,
		] );
		$result = [];
		while ( $query->have_posts() ) {
			$query->the_post();
			$content = self::get_content();
			$word_count = self::get_words_count( $content );
			/* cache the data for two weeks */
			set_transient( 'tbp_readtime_' . get_the_ID() . get_post_timestamp( null, 'modified' ), $word_count, DAY_IN_SECONDS * 14 );
			$result[ get_the_ID() ] = $word_count;
		}

		wp_send_json_success( $result );
	}

	public static function clear_cache( $post_id ) {
		delete_transient( 'tbp_readtime_' . $post_id . get_post_timestamp( $post_id, 'modified' ) );
	}
}

if ( method_exists( 'Themify_Builder_Model', 'add_module' ) ) {
    new TB_Read_Time_Module();
} else {
    Themify_Builder_Model::register_module( 'TB_Read_Time_Module' );
}