<?php
defined( 'ABSPATH' ) || exit; // Exit if accessed directly

/**
 * Module Name: Statistics
 * Description: 
 */
class TB_Statistics_Module extends Themify_Builder_Component_Module {

	public function __construct() {
            if(method_exists('Themify_Builder_Model', 'add_module')){
                parent::__construct('stats');
            }
            else{//backward
                 parent::__construct(array(
                    'name' =>$this->get_name(),
                    'slug' => 'stats',
                    'category' =>$this->get_group()
                ));
            }
	}
        
	public function get_name(){
            return __('Statistics', 'tbp');
        }
        
	public function get_icon(){
		return 'bar-chart';
	}
        
        public function get_group() {
            return array( 'site');
        }

	public function get_options() {
		return [
			[
				'id' => 's',
				'label' => __( 'Items', 'tbp' ),
			    'type' => 'sortable_fields',
				'options' => [
				    'posts' => [
					    'label' => __( 'Posts', 'tbp' ),
						'has_global_options' => true,
						'options' => [
							[
								'label' => __( 'Post Type', 'tbp' ),
								'id' => 'post_type',
								'type' => 'select',
								'options' => Themify_Builder_Model::get_public_post_types(),
							],
						],
					],
				    'comments' => [
					    'label' => __( 'Comments', 'tbp' ),
						'has_global_options' => true,
						'options' => [
							[
								'label' => __( 'Status', 'tbp' ),
								'id' => 'status',
								'type' => 'select',
								'options' => [
									'' => '',
									'approved'  => __( 'Approved', 'tbp' ),
									'spam'      => __( 'Spam', 'tbp' ),
									'trash'     => __( 'Trash', 'tbp' ),
									'moderated' => __( 'Moderated', 'tbp' ),
								],
							],
						],
					],
				    'terms' => [
					    'label' => __( 'Terms', 'tbp' ),
						'has_global_options' => true,
						'options' => [
							array(
								'label' => __( 'Taxonomy', 'tbp' ),
								'id' => 'tax',
								'type' => 'select',
								'options' => Themify_Builder_Model::get_public_taxonomies(),
							),
							array(
								'label' => __( 'Hide Empty', 'tbp' ),
								'id' => 'hide_empty',
								'type' => 'select',
								'options' => array(
									1 => __( 'Yes', 'tbp' ),
									0 => __( 'No', 'tbp' ),
								),
							)
						],
					],
				    'users' => [
					    'label' => __( 'Users', 'tbp' ),
						'has_global_options' => true,
						'options' => [
							array(
								'label' => __( 'Role', 'tbp' ),
								'id' => 'role',
								'type' => 'select',
								'options' => self::get_roles(),
								'help' => __( 'Leave empty to count all users regardless of their role.', 'tbp' ),
							)
						],
					],
					'text' => [
						'label' => __( 'Plain Text', 'tbp' ),
					],
				],
			],
			[
				'id' => 'tag',
				'label' => __( 'Display', 'tbp' ),
				'type' => 'select',
				'options' => [
					'ul' => __( 'Unordered List', 'tbp' ),
					'ol' => __( 'Ordered List', 'tbp' ),
					'p' => __( 'Paragraphs', 'tbp' ),
					'span' => __( 'Inline', 'tbp' ),
				],
				'binding' => [
					'not_empty' => [ 'hide' => 'sep' ],
					'span' => [ 'show' => 'sep' ],
				],
			],
			[
				'id' => 'sep',
				'type' => 'text',
				'class' => 'xsmall',
				'label' => __( 'Separator', 'tbp' ),
			],
			[ 'type' => 'custom_css_id', 'custom_css' => 'css' ],
		];
	}

	public static function get_roles() {
		$editable_roles = get_editable_roles();
		$roles = array( '' => '' );
		foreach ( $editable_roles as $role => $details ) {
			$roles[ $role ] = translate_user_role( $details['name'] );
		}

		return $roles;
	}

	public function get_styling() {
		$general = array(
			// Background
			self::get_expand('bg', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_image('', 'b_i','bg_c','b_r','b_p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_image('', 'b_i','bg_c','b_r','b_p', 'h')
					)
					)
				))
			)),
			// Font
			self::get_expand('f', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_font_family('', 'f_f'),
						self::get_color_type( '','', 'f_c_t',  'f_c', 'f_g_c'),
						self::get_font_size('', 'f_s'),
						self::get_line_height('', 'l_h'),
						self::get_letter_spacing('', 'l_s'),
						self::get_text_align( '', 't_a'),
						self::get_text_transform('', 't_t'),
						self::get_font_style('', 'f_st', 'f_w'),
						self::get_text_decoration('', 't_d_r'),
						self::get_text_shadow('','t_sh'),
					)
					),
					'h' => array(
					'options' => array(
						self::get_font_family('', 'f_f_h'),
						self::get_color_type( '','', 'f_c_t_h',  'f_c_h', 'f_g_c_h'),
						self::get_font_size('', 'f_s', '', 'h'),
						self::get_font_style('', 'f_st', 'f_w', 'h'),
						self::get_text_decoration('', 't_d_r', 'h'),
						self::get_text_shadow('','t_sh','h'),
					)
					)
				))
			)),
			// Link
			self::get_expand('l', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_color_type( '','', 'l_c_t',  'l_c', 'l_g_c'),
						self::get_text_decoration( '', 't_d')
					)
					),
					'h' => array(
					'options' => array(
						self::get_color_type( '','', 'l_c_t_h',  'l_c_h', 'l_g_c_h'),
						self::get_text_decoration( '', 't_d', 'h')
					)
					)
				))
			)),
			// Padding
			self::get_expand('p', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_padding('', 'p')
					)
					),
					'h' => array(
					'options' => array(
						self::get_padding('', 'p', 'h')
					)
					)
				))
			)),
			// Margin
			self::get_expand('m', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_margin('', 'm')
					)
					),
					'h' => array(
					'options' => array(
						self::get_margin('', 'm', 'h')
					)
					)
				))
			)),
			// Border
			self::get_expand('b', array(
				self::get_tab(array(
					'n' => array(
					'options' => array(
						self::get_border('', 'b')
					)
					),
					'h' => array(
					'options' => array(
						self::get_border('', 'b', 'h')
					)
					)
				))
			)),
			// Filter
			self::get_expand('f_l',
				array(
					self::get_tab(array(
						'n' => array(
							'options' => count($a = self::get_blend())>2 ? array($a) : $a
						),
						'h' => array(
							'options' => count($a = self::get_blend('','bl_m_h','h'))>2 ? array($a + array('ishover'=>true)) : $a
						)
					))
				)
			),
			// Width
			self::get_expand('w', array(
				self::get_width('', 'w')
			)),
			! method_exists( $this, 'get_max_height' ) ? array() :
			// Height & Min Height
			self::get_expand('ht', array(
				self::get_height(),
				self::get_min_height(),
				self::get_max_height()
			)),
			// Rounded Corners
			self::get_expand('r_c', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_border_radius('', 'r_c')
						)
					),
					'h' => array(
						'options' => array(
							self::get_border_radius('', 'r_c', 'h')
						)
					)
				))
			)),
			// Shadow
			self::get_expand('sh', array(
				self::get_tab(array(
					'n' => array(
						'options' => array(
							self::get_box_shadow('', 'sh')
						)
					),
					'h' => array(
						'options' => array(
							self::get_box_shadow('', 'sh', 'h')
						)
					)
				))
			)),
		);

		return array(
			'type' => 'tabs',
			'options' => array(
				'g' => array(
					'options' => $general
				),
			)
		);
	}

	public function get_live_default() {
		return [
			'sep' => ', ',
		];
	}

	public function get_category() {
		return array( 'site' );
	}
}
if(method_exists('Themify_Builder_Model', 'add_module')){
    new TB_Statistics_Module();
}
else{
    Themify_Builder_Model::register_module('TB_Statistics_Module');
}